#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Copyright (c) 2008 ADITAM project (contact@aditam.org)
#    License: GNU GPLv3
#
#    This file is part of the ADITAM project.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.  See the GNU General Public License
#    for more details.
#

__author__ = '$LastChangedBy: jschneider $'
__date__ = '$LastChangedDate: 2008-06-01 21:09:01 +0200 (Sun, 01 Jun 2008) $'
__version__ = '$Rev: 223 $'

import logging
import time
import Pyro.core

from threading import Thread
from aditam.agent.utils import autoreconnect
from Pyro.errors import ConnectionClosedError

class Monitoring(Thread):
    def __init__(self, uri_server, hostname):
        """
        uri_server : Pyro uri of the server obj
        hostname : name of the agent
        """
        Thread.__init__(self)
        self.server = Pyro.core.getProxyForURI(uri_server)
        self.hostname = hostname

    def run(self):
        """
        Launch the loop
        """
        logging.debug("Starting the monitoring ...")
        while 1:
            logging.debug("Send up to the server")
            autoreconnect(self.server, self.server.agent_up, self.hostname)
            time.sleep(8)
 



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Copyright (c) 2008 ADITAM project (contact@aditam.org)
#    License: GNU GPLv3
#
#    This file is part of the ADITAM project.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.  See the GNU General Public License
#    for more details.
#

___author__ = '$LastChangedBy$'
__date__ = '$LastChangedDate$'
__version__ = '$Rev$'

import Pyro.core
import Pyro.naming
import Pyro.util
import Pyro.errors
import platform
import sys
import socket
import logging
import time

from aditam.agent.jobmanager import JobManager
from aditam.agent.monitoring import Monitoring
from aditam.core.config import agent_config
from aditam.core.errors import ConfigError

class Manager:

    def __check_config(self):
        """Test if the agent configuration is full"""
        try:
            agent_config.need_section('server')
            agent_config.need_option('server', 'host')
            agent_config.need_section('logger')
            agent_config.need_option('logger', 'level')
            agent_config.need_option('logger', 'filename')
        except ConfigError, e:
            logging.critical(e)
            raise SystemExit, e

    def __init_logger(self):
        """Set the default logger of the agent"""
        level = agent_config.get('logger', 'level')
        if level == "DEBUG":
            level = logging.DEBUG
        elif level == "WARNING":
            level = logging.WARNING
        elif level == "ERROR":
            level = logging.ERROR
        elif level == "CRITICAL":
            level = logging.CRITICAL
        else:
            level = logging.INFO
        filename = agent_config.get('logger', 'filename')
        logging.basicConfig(level=level,
                format="%(asctime)s %(name)-12s %(levelname)-8s %(message)s",
                filename=filename,
                filemode='a'
                )
        # define a Handler which writes messages to the sys.stderr
        console = logging.StreamHandler()
        console.setLevel(level)
        formatter = logging.Formatter("%(levelname)-8s %(message)s")
        console.setFormatter(formatter)
        logging.getLogger('').addHandler(console)

    def __get_uri(self, host, port, obj_name):
        """ """
        if not host:
            host = agent_config.get('server', 'host')
        if not host:
            err = "No host define ! Please use aditam-admin.py -a or " +\
                    "aditam-agent.py --host"
            logging.critical(err)
            raise SystemExit, err
        if not port and agent_config.has_option('server', 'port'):
            port = agent_config.get('server', 'port')
        if not port:
            return 'PYROLOC://' + host + '/' + obj_name
        else:
            return 'PYROLOC://' + host + ':'+ port + '/' + obj_name

    def start(self, host=None, port=0):
        """Launch the agent"""
        self.__check_config()
        self.__init_logger()
        os = platform.system()
        # initialize the client
        Pyro.core.initClient()
        uri_server = self.__get_uri(host, port, 'server')
        uri_feedback = self.__get_uri(host, port, 'feedback')
        server = Pyro.core.getAttrProxyForURI(uri_server)

        if agent_config.has_option('agent', 'hostname') and\
                agent_config.get('agent', 'hostname'):
            hostname = agent_config.get('agent', 'hostname')
        else:
            hostname = socket.getfqdn()

        try:
            server.add_agent(unicode(hostname), os)
        except Pyro.errors.ProtocolError,x:
            logging.critical("Server is unreachable : %s" % x)
            raise SystemExit, x
        jobmanager = JobManager(uri_server, uri_feedback, hostname)
        jobmanager.start()
        monitoring = Monitoring(uri_server, hostname)
        monitoring.start()
        logging.info('Agent is started.')
        while 42:
            try:
                time.sleep(42)
            except KeyboardInterrupt:
                logging.info("Keyboard interruption")
                self.stop()

    def stop(self):
        """
        Stop the farmnode
        """
        logging.info("Stop Aditam agent")
        sys.exit(0)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Copyright (c) 2008 ADITAM project (contact@aditam.org)
#    License: GNU GPLv3
#
#    This file is part of the ADITAM project.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.  See the GNU General Public License
#    for more details.
#

___author__ = '$LastChangedBy: jschneider $'
__date__ = '$LastChangedDate: 2008-06-01 21:09:01 +0200 (Sun, 01 Jun 2008) $'
__version__ = '$Rev: 223 $'

import os
import sys

from optparse import OptionParser

from aditam.agent.manager import Manager
from aditam.core.config import agent_config

parser = OptionParser()

parser.add_option("-p", "--port",
        dest="port",
        metavar="<port>",
        help="Listen port",
        type="int")
parser.add_option("-H", "--host",
        dest="host",
        metavar="<host>",
        help="Listen host",
        type="string")

def main():
    if os.name == 'posix':
        from aditam.core.daemon import Daemon
        daemon = Daemon(
                stderr=agent_config.get("logger", "filename"),
                pidfile="/var/run/aditam-agent.pid",
                )
        if daemon.service(parser):
            (options, args) = (daemon.options, daemon.args)
            manager = Manager()
            manager.start(options.host, options.port)
    else:
        (options, args) = parser.parse_args()
        manager = Manager()
        manager.start(options.host, options.port)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Copyright (c) 2008 ADITAM project (contact@aditam.org)
#    License: GNU GPLv3
#
#    This file is part of the ADITAM project.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.  See the GNU General Public License
#    for more details.
#

___author__ = '$LastChangedBy: jschneider $'
__date__ = '$LastChangedDate: 2008-05-31 10:34:49 +0200 (Sat, 31 May 2008) $'
__version__ = '$Rev: 211 $'

import sys
import os

from setuptools import setup, find_packages

# Project uses reStructuredText, so ensure that the docutils get
# installed or upgraded on the target machine
install_requires = [
    "Pyro >= 0.7",
    "aditam.core >= 0.1",
    "setuptools >= 0.6c7"
]

README = open(os.path.join(os.path.dirname(__file__),
    'README.txt')).read()

setup(
    name = "aditam.agent",
    version = "0.1",
    include_package_data=True,
    install_requires = install_requires,
    scripts = ['aditam-agent.py'],
    zip_safe = False,
    packages = find_packages(),
    namespace_packages = ['aditam'],
    author = "ADITAM project",
    author_email = "contact@aditam.org",
    description = "Automated and DIstributed TAsk Manager agent part.",
    license = "GPLv3",
    keywords = "tasks",
    classifiers=[
          "Development Status :: 2 - Pre-Alpha",
          "Environment :: Console",
          "Intended Audience :: System Administrators",
          "Intended Audience :: Other Audience",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
      ],
    long_description=README,
    url = "http://www.aditam.org/",
    test_suite = 'nose.collector'
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Copyright (c) 2008 ADITAM project (contact@aditam.org)
#    License: GNU GPLv3
#
#    This file is part of the ADITAM project.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.  See the GNU General Public License
#    for more details.
#

___author__ = '$LastChangedBy: jschneider $'
__date__ = '$LastChangedDate: 2008-06-01 21:09:01 +0200 (Sun, 01 Jun 2008) $'
__version__ = '$Rev: 223 $'


import Pyro.core
import Pyro.errors
import time
import logging

from Queue import Queue
from threading import Thread
from subprocess import Popen, PIPE
from Pyro.errors import ConnectionClosedError

from aditam.core.config import agent_config
from aditam.agent.utils import autoreconnect

class JobManager(Thread):
    """ Excute the jobs and send reports. """

    def __init__(self, uri_server, uri_feedback, hostname):
        """
        uri_server : Pyro uri of the server obj
        uri_feedback : Pyro uri of the feedback obj
        hostname : name of the agent
        """
        Thread.__init__(self)
        self.hostname = hostname
        self.server = Pyro.core.getProxyForURI(uri_server)
        self.feedback = Pyro.core.getProxyForURI(uri_feedback)

    def _is_command_ok(self, returncode):
        """
        Test if the command was execute succesfully.
        Return a boolean.
        """
        if not returncode:
            return True
        return False

    def unicode_encode(self, text, charsets):
        """
        Try to encode a string in unicode
        If the encoding failled this method return
        the original text
        """
        for charset in charsets:
            try:
                res = unicode(text, charset)
                return res
            except UnicodeDecodeError:
                pass
        logging.warning("unicode encode failled for '%s'" % text)
        return text

    def execute_job(self, agentjob):
        """
        Executes the commands and send a command report
        agentjob : an AgentJob object
        command_report : {'cmd_id', 'stdout', 'stderr', 'result'}
        """
        logging.info("Execute new job %d", agentjob.get_job_id())
        for command in agentjob.get_commands():
            command_report = dict(cmd_id=command['id'])
            p = Popen(command['command_line'],
                    shell=True,
                    stdin=PIPE,
                    stdout=PIPE,
                    stderr=PIPE)
            returncode = p.wait()
            stdout = self.unicode_encode(p.stdout.read(), ('utf-8', 'iso8859-15'))
            stderr = self.unicode_encode(p.stderr.read(), ('utf-8', 'iso8859-15'))
            p.stdin.close()
            p.stdout.close()
            p.stderr.close()
            command_report['stdout'] = stdout
            command_report['stderr'] = stderr
            command_report['result'] = returncode
            autoreconnect(self.feedback, self.feedback.add_command_report,
                    command_report, agentjob.get_job_id())

            if not self._is_command_ok(returncode):
                logging.warning("Command '%s' failled !" % command['command_line'])
                autoreconnect(self.feedback, self.feedback.end_job,
                        agentjob.get_job_id(), False)
                return None
        autoreconnect(self.feedback, self.feedback.end_job, agentjob.get_job_id())

    def run(self):
        """
        Loop which check new job.
        """
        logging.debug("Starting the jobmanager ...")
        while 1:
            logging.debug("Ask a job to the server")
            agentjob = autoreconnect(self.server, self.server.get_job, self.hostname)
            if agentjob:
                self.execute_job(agentjob)
            time.sleep(10)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    Copyright (c) 2008 ADITAM project (contact@aditam.org)
#    License: GNU GPLv3
#
#    This file is part of the ADITAM project.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.  See the GNU General Public License
#    for more details.
#

__author__ = '$LastChangedBy: schnei_e@EPITECH.NET $'
__date__ = '$LastChangedDate: 2008-10-10 23:28:05 +0200 (ven 10 oct 2008) $'
__version__ = '$Rev: 188 $'

import logging
import time
import sys
import Pyro.errors

from aditam.core.config import agent_config

"""
Common function for the agent
"""

def autoreconnect(obj, method_call, *method_vars):
    """
    If the server is down the agent try to reconnect
    indefinitely.
    obj: pyro object
    Example :
    obj.foo("bar1", "bar2") => autoreconnect(obj, obj.foo, "bar1", "bar2")
    """
    try:
        return method_call(*method_vars)
    except Pyro.errors.ConnectionClosedError, x:
        logging.warning("Server is unreachable.")
        while 1:
            try:
                obj.adapter.bindToURI(obj.URI)
                logging.info("Server is back")
                return method_call(*method_vars)
            except Pyro.errors.ProtocolError:
                time.sleep(1)

